#!/bin/bash

# This script installs ICARUS Verilog if $ICARUS_HOME is not set,
#                      Veritool if $VERITOOL_HOME is not set

if [ "x" == "x$ISPRAS_HOME" ]
then
  ISPRAS_HOME=$HOME
  export ISPRAS_HOME=$ISPRAS_HOME
fi

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --force )
      FORCE=1
    ;;
    * )
  esac
  shift
done

if [ "x" == "x$ICARUS_HOME" ]
then
  INSTALL_ICARUS=1
  ICARUS_HOME="$ISPRAS_HOME/tools/verilog"
  UPDATE_PROFILE_ICARUS=1
else if [ "x" == "x$FORCE" ]
then
  tput setaf 3
  echo "[WARNING] Environment variable ICARUS_HOME has been already set. It might point to the fact that Icarus Verilog has been already installed."
  echo "[WARNING] If you want Icarus Verilog to be reinstalled, type parameter --force-install-veritool"
  tput sgr0
else
  INSTALL_ICARUS=1
fi
fi

if [ "x" == "x$VERITOOL_HOME" ]
then
  INSTALL_VERITOOL=1
  VERITOOL_HOME="$ISPRAS_HOME/tools/veritool"
  UPDATE_PROFILE_VERITOOL=1
else if [ "x" == "x$FORCE" ]
then
  tput setaf 3
  echo "[WARNING] Environment variable VERITOOL_HOME has been already set. It might point to the fact that VeriTool has been already installed."
  echo "[WARNING] If you want VeriTool to be reinstalled, type parameter --force-install-veritool"
  tput sgr0
else 
  INSTALL_VERITOOL=1
fi
fi

##################
# Icarus Verilog #
##################

if [ "x" != "x$INSTALL_ICARUS" ]
then
  CURR_DIR=`pwd`
  ICARUS_DIR="./downloads/sources/verilog"
  #ICARUS_HOME="$ISPRAS_HOME/tools/verilog"
  ICARUS_VER="0.9.4"
  mkdir -p "$ICARUS_DIR"
  mkdir -p "$ICARUS_HOME"
  cd "$ICARUS_DIR"
  rm -f *tar.gz*
  wget -q "http://sourceforge.net/projects/iverilog/files/iverilog/$ICARUS_VER/verilog-$ICARUS_VER.tar.gz"
  RESULT_OF_WGET=`find . -name verilog-$ICARUS_VER.tar.gz`
  if [ -n "$RESULT_OF_WGET" ]
  then
    tar xf verilog-$ICARUS_VER.tar.gz
    cd verilog-$ICARUS_VER
    ./configure --prefix="$ICARUS_HOME"
    make && make install
    export ICARUS_HOME=$ICARUS_HOME
    if [ "x" != "x$UPDATE_PROFILE_ICARUS" ]
    then
      echo "export ICARUS_HOME=$ICARUS_HOME" >> "$HOME/.profile"
      echo "export ICARUS_HOME=$ICARUS_HOME" >> "$HOME/.bashrc"
    fi
    cd $CURR_DIR
    tput setaf 2
    echo "[OK] Icarus Verilog has been installed..."
    tput sgr0
  else
    tput setaf 1
    echo "[FAILED] Icarus Verilog has NOT been installed. Check your Internet connection or try to install the program manually!"
    tput sgr0
  fi
fi

############
# VeriTool #
############

if [ "x" != "x$INSTALL_VERITOOL" ]
then
  VERITOOL_DIR="./downloads/sources/veritool"
  #VERITOOL_HOME="$ISPRAS_HOME/tools/veritool"
  VERITOOL_VER="0.1-beta-20110614"
  mkdir -p "$VERITOOL_DIR"
  mkdir -p "$VERITOOL_HOME"
  cd "$VERITOOL_DIR"
  rm -f *tar.gz*
  wget -q "http://forge.ispras.ru/attachments/download/898/veritool-$VERITOOL_VER.tar.gz"
  RESULT_OF_WGET=`find . -name veritool-$VERITOOL_VER.tar.gz`
  if [ -n "$RESULT_OF_WGET" ]
  then
    tar xf veritool-$VERITOOL_VER.tar.gz
    cd "veritool-0.1-beta"
    ./configure --prefix="$VERITOOL_HOME"
    make all && make install
    if [ "x" != "x$UPDATE_PROFILE_VERITOOL" ]
    then
      echo "export VERITOOL_HOME=$VERITOOL_HOME" >> "$HOME/.profile"
      echo "export VERITOOL_HOME=$VERITOOL_HOME" >> "$HOME/.bashrc"
    fi
    tput setaf 2
    echo "[OK] VeriTool has been installed..."
    tput sgr0
  else
    tput setaf 1
    echo "[FAILED] VeriTool has NOT been installed. Check your Internet connection or try to install the program manually!"
    tput sgr0
  fi
fi

