/** @file multififo_media.h
 * Header for the mediator of united FIFO's model.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/media.hpp>
#include <multififo_model.h>
#include <interface.h>
#include <fifo_interface.h>

#include <string>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief Mediator for Multififo model.
 */
CPPTESK_MEDIA(MultififoMediator, Multififo)
{
public:
    MultififoMediator();
    virtual ~MultififoMediator();

    /// Max delay time of registered reactions (in cycles).
    const static int REACTION_TIMEOUT = 100;

    virtual bool isIface1aReady() const;
    virtual bool isIface1bReady() const;
    virtual bool isIface2Ready() const;

    /// Input interface iface1a serializer.
    CPPTESK_DECLARE_STIMULUS(serialize_iface1a);
    /// Input interface iface1b serializer.
    CPPTESK_DECLARE_STIMULUS(serialize_iface1b);
    /// Input interface iface2 serializer.
    CPPTESK_DECLARE_STIMULUS(serialize_iface2);

    /// Output interface iface3 deserializer.
    CPPTESK_DECLARE_REACTION(deserialize_iface3);

    CPPTESK_DECLARE_PROCESS(deserialize_inner_iface0);
    CPPTESK_DECLARE_PROCESS(deserialize_inner_iface1);

    /// Detector of unexpected reactions for output iface3.
    CPPTESK_DEFINE_BASIC_OUTPUT_LISTENER(listen_iface3, iface3, outputs.DO_VAL);

    /// Arbiter of output interface to subdivide reactions on the different cycles.
    CPPTESK_DECLARE_ARBITER(Priority, matcher_iface3);

    /// Initialization of test environment.
    virtual void initialize();

    /// Finalization of test process.
    virtual void finalize();

    /// Set input values to the DUT.
    virtual void setInputs();

    /// Get output values from the DUT.
    virtual void getOutputs();

    /// Simulate one cycle of DUT's work.
    virtual void simulate();

    virtual void cycle();
private:
    /// Structure with input signals of DUT.
    inputs_t inputs;
    /// Structure with output signals of DUT.
    outputs_t outputs;
};

}}}
