/** @file fifo_scen.cpp
 * Scenarios of Multififo verification.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <fifo_scen.h>

#include <tracer/tracer.hpp>

namespace cpptesk {
namespace examples {
namespace fifo {

bool FIFOScenario::init(int argc, char ** argv)
{
    return true;
}

void FIFOScenario::finish()
{
    dut.finalize();
}

bool FIFOScenario::scen_nop(Ctx& ctx)
{
    IBEGIN
    IACTION
    {
        dut.cycle();
        YIELD(dut.verdict());
    }
    IEND
}

bool FIFOScenario::scen_push_a(Ctx& ctx)
{
    IBEGIN
    IACTION
    {
        if(dut.isIface1aReady() && !dut.fifo0.is_full() && !dut.fifo2.is_full()) {
            InputData msg = InputData();
            dut.start(&Multififo::push_msg_a, dut.iface1a, msg, Multififo::START_REQUEST_IFACE);
            //dut.fifo0.start(&FIFO::pop_msg, dut.fifo0.iface2, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

bool FIFOScenario::scen_push_b(Ctx& ctx)
{
    IBEGIN
    IACTION
    {
        if(dut.isIface1bReady() && !dut.fifo1.is_full() && !dut.fifo2.is_full()) {
            InputData msg = InputData();
            dut.start(&Multififo::push_msg_b, dut.iface1b, msg, Multififo::START_REQUEST_IFACE);
            //dut.fifo1.start(&FIFO::pop_msg, dut.fifo1.iface2, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

bool FIFOScenario::scen_pop(cpptesk::ts::IntAbcCtx& ctx)
{
    IBEGIN
    IACTION
    {
        if(dut.isIface2Ready() && !dut.fifo2.is_empty()) {
            InputData msg = InputData();
            dut.start(&Multififo::pop_msg, dut.iface2, msg, Multififo::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

std::string FIFOScenario::get_state_fifo()
{
    std::string state;
    std::stringstream out;

    out << "{fifo:" << dut.fifo0.fifo_fulness() << "," << dut.fifo1.fifo_fulness() << "," << dut.fifo2.fifo_fulness() 
        << ", write1_started:" << dut.isIface1aReady() << ", write2_started:" << dut.isIface1bReady() << ", read_started:" << dut.isIface2Ready() << "}";
    state = out.str();
#ifdef DEBUG
    std::cout << "get_state_fifo(): " << state << std::endl;
#endif
    return state;
}

FIFOScenario::FIFOScenario(void) : ScenarioBase()
{
}
FIFOScenario::~FIFOScenario(void) {}

ExampleScenario::ExampleScenario(void) : FIFOScenario()
{
    setup("MultiFIFOScenario", &FIFOScenario::init, &FIFOScenario::finish, &FIFOScenario::get_state_fifo);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_nop);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_push_a);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_push_b);
    CPPTESK_ADD_SCENARIO_METHOD(FIFOScenario::scen_pop);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

