/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Assertion macros
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/debug.hpp>

#include <iostream>
#include <assert.h>

/** @def CPPTESK_ASSERTION
 * C++TESK assertion macros.
 */
#define CPPTESK_ASSERTION(condition, message) \
    __CPPTESK_ASSERTION_FILE_LINE(condition, message, __FILE__, __LINE__)

/** @def CPPTESK_ASSERTION_FILE_LINE
 * Extended C++TESK assertion macros.
 */
#define __CPPTESK_ASSERTION_FILE_LINE(condition, message, file, line) \
    do { \
        if(!(condition)) { \
            __CPPTESK_DEBUG_PRINT_FILE_LINE(::cpptesk::hw::DEBUG_ERROR, message, file, line); \
            assert(false); \
        } \
    } while(false)

#ifdef HW_SHORT_NAMES
    #define ASSERTION(condition, message) CPPTESK_ASSERTION(condition, message)
#endif // HW_SHORT_NAMES

